// string-or-int.scalar.ts
import { Scalar, CustomScalar } from '@nestjs/graphql';
import { Kind, ValueNode } from 'graphql';

@Scalar('StringOrInt')
export class StringOrIntScalar
  implements CustomScalar<string | number, string | number>
{
  description = 'StringOrInt custom scalar type';

  parseValue(value: string | number): string | number {
    if (typeof value === 'string' || typeof value === 'number') {
      return value;
    }
    throw new Error('StringOrInt must be either a string or a number');
  }

  serialize(value: string | number): string | number {
    return value;
  }

  parseLiteral(ast: ValueNode): string | number {
    if (
      ast.kind === Kind.STRING ||
      ast.kind === Kind.INT ||
      ast.kind === Kind.FLOAT
    ) {
      return ast.value;
    }
    throw new Error('StringOrInt must be either a string or a number');
  }
}
