import {
  ExceptionFilter,
  Catch,
  ArgumentsHost,
  HttpException,
  Logger,
} from '@nestjs/common';
import { GqlArgumentsHost } from '@nestjs/graphql';
import { GraphQLError } from 'graphql';

@Catch()
export class GqlExceptionFilter implements ExceptionFilter {
  private readonly logger = new Logger(GqlExceptionFilter.name);

  catch(exception: unknown, host: ArgumentsHost) {
    const gqlHost = GqlArgumentsHost.create(host);

    const context = gqlHost.getContext();
    const response = context.res;
    const status =
      exception instanceof HttpException ? exception.getStatus() : 500;

    const errorResponse = {
      statusCode: status,
      message:
        exception instanceof HttpException
          ? exception.getResponse()
          : 'Internal server error',
    };

    this.logger.error(`Exception caught: ${JSON.stringify(errorResponse)}`);

    if (response) {
      response.status(status).json(errorResponse);
    } else {
      // 如果没有 response 对象，例如在 GraphQL 请求中
      return new GraphQLError(errorResponse.message.toString(), {
        extensions: {
          code: 'INTERNAL_SERVER_ERROR',
          exception: errorResponse,
        },
      });
    }
  }
}
